@@ -743,7 +743,7 @@ static void _calculate_timeout(NotifyDaemon* daemon, NotifyTimeout* nt, int time
 
 		theme_set_notification_timeout(nt->nw, timeout);
 
-		glong usec = timeout * 1000;  /* convert from msec to usec */
+		glong usec = timeout * 1000L;  /* convert from msec to usec */
 
 		/*
 		 * If it's less than 0, wrap around back to MAXLONG.
@@ -955,9 +955,12 @@ static GdkPixbuf* _notify_daemon_pixbuf_from_path(const char* path)
 		if (!strncmp (path, "file://", 7))
 		{
 			path += 7;
+
+			/* Unescape URI-encoded, allowed characters */
+			path = g_uri_unescape_string (path, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH);
 		}
 
-			/* Load file */
+		/* Load file */
 		pixbuf = gdk_pixbuf_new_from_file (path, NULL);
 	}
 	else
